@@ -41,6 +41,91 @@ except ImportError:
 else:
     my_decorator = decorator_module.decorator
 
+__doc__ = """
+A Collection of Utilities
+=========================
+
+Math
+----
+
+.. autofunction:: levi_civita
+.. autofunction:: perm
+.. autofunction:: comb
+
+Assertive accessors
+-------------------
+
+.. autofunction:: one
+.. autofunction:: is_single_valued
+.. autofunction:: all_roughly_equal
+.. autofunction:: single_valued
+
+Memoization
+-----------
+
+.. autofunction:: memoize
+.. autofunction:: memoize_on_first_arg
+.. autofunction:: memoize_method
+.. autofunction:: memoize_method_with_uncached
+.. autofunction:: memoize_in
+
+Argmin/max
+----------
+
+.. autofunction:: argmin2
+.. autofunction:: argmax2
+.. autofunction:: argmin
+.. autofunction:: argmax
+
+Cartesian products
+------------------
+.. autofunction:: cartesian_product
+.. autofunction:: distinct_pairs
+
+Permutations, Tuples, Integer sequences
+---------------------------------------
+
+.. autofunction:: wandering_element
+.. autofunction:: indices_in_shape
+.. autofunction:: generate_nonnegative_integer_tuples_below
+.. autofunction:: generate_nonnegative_integer_tuples_summing_to_at_most
+.. autofunction:: generate_all_nonnegative_integer_tuples
+.. autofunction:: generate_all_integer_tuples_below
+.. autofunction:: generate_all_integer_tuples
+.. autofunction:: generate_permutations
+.. autofunction:: generate_unique_permutations
+
+Graph Algorithms
+----------------
+
+.. autofunction:: a_star
+
+Formatting
+----------
+
+.. autoclass:: Table
+.. autofunction:: string_histogram
+.. autofunction:: word_wrap
+
+Debugging
+---------
+
+.. autofunction:: typedump
+.. autofunction:: invoke_editor
+
+Progress bars
+-------------
+
+.. autoclass:: ProgressBar
+
+Name generation
+---------------
+
+.. autofunction:: generate_unique_names
+.. autofunction:: generate_numbered_unique_names
+.. autofunction:: UniqueNameGenerator
+"""
+
 
 # {{{ math --------------------------------------------------------------------
 
@@ -197,6 +282,19 @@ class Record(RecordWithoutPickling):
     def __ne__(self, other):
         return not self.__eq__(other)
 
+
+class ImmutableRecordWithoutPickling(RecordWithoutPickling):
+    "Hashable record. Does not explicitly enforce immutability."
+
+    def __hash__(self):
+        return hash(
+                (type(self),) + tuple(getattr(self, field)
+                    for field in self.__class__.fields))
+
+
+class ImmutableRecord(ImmutableRecordWithoutPickling, Record):
+    pass
+
 # }}}
 
 
@@ -441,6 +539,7 @@ def memoize(*args, **kwargs):
     raise TypeError(
         "memoize received unexpected position arguments: %s" % args)
 
+
 FunctionValueCache = memoize
 
 
@@ -1120,6 +1219,7 @@ def generate_all_nonnegative_integer_tuples(length, least=0):
                     yield prebase + [current_max] + postbase
         current_max += 1
 
+
 # backwards compatibility
 generate_positive_integer_tuples_below = generate_nonnegative_integer_tuples_below
 generate_all_positive_integer_tuples = generate_all_nonnegative_integer_tuples
@@ -1148,7 +1248,7 @@ def generate_all_integer_tuples(length, least_abs=0):
 
 
 def generate_permutations(original):
-    """Generate all permutations of the list `original'.
+    """Generate all permutations of the list *original*.
 
     Nicked from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/252178
     """
@@ -1162,7 +1262,7 @@ def generate_permutations(original):
 
 
 def generate_unique_permutations(original):
-    """Generate all unique permutations of the list `original'.
+    """Generate all unique permutations of the list *original*.
     """
 
     had_those = set()
@@ -1299,7 +1399,13 @@ def a_star(initial_state, goal_state, neighbor_map,
 # {{{ table formatting
 
 class Table:
-    """An ASCII table generator."""
+    """An ASCII table generator.
+
+    .. automethod:: add_row
+    .. automethod:: __str__
+    .. automethod:: latex
+    """
+
     def __init__(self):
         self.rows = []
 
@@ -1387,10 +1493,10 @@ def string_histogram(iterable, min_value=None, max_value=None,
 
 def word_wrap(text, width, wrap_using="\n"):
     # http://code.activestate.com/recipes/148061-one-liner-word-wrap-function/
-    """
+    r"""
     A word-wrap function that preserves existing line breaks
     and most spaces in the text. Expects that existing line
-    breaks are posix newlines (\n).
+    breaks are posix newlines (``\n``).
     """
     space_or_break = [" ", wrap_using]
     return reduce(lambda line, word, width=width: '%s%s%s' %
@@ -1586,6 +1692,14 @@ def invoke_editor(s, filename="edit.txt", descr="the file"):
 # {{{ progress bars
 
 class ProgressBar:
+    """
+    .. automethod:: draw
+    .. automethod:: progress
+    .. automethod:: set_progress
+    .. automethod:: finished
+    .. automethod:: __enter__
+    .. automethod:: __exit__
+    """
     def __init__(self, descr, total, initial=0, length=40):
         import time
         self.description = descr
@@ -1734,18 +1848,48 @@ def generate_unique_names(prefix):
         yield "%s_%d" % (prefix, try_num)
         try_num += 1
 
+
+def generate_numbered_unique_names(prefix, num=None):
+    orig_num = num
+    num = 0
+    if orig_num is None:
+        yield (num, prefix)
+
+    while True:
+        name = "%s_%d" % (prefix, num)
+        num += 1
+        yield (num, name)
+
+
 generate_unique_possibilities = MovedFunctionDeprecationWrapper(
         generate_unique_names)
 
 
-class UniqueNameGenerator:
+class UniqueNameGenerator(object):
+    """
+    .. automethod:: is_name_conflicting
+    .. automethod:: add_name
+    .. automethod:: add_names
+    .. automethod:: __call__
+    """
     def __init__(self, existing_names=set(), forced_prefix=""):
         self.existing_names = existing_names.copy()
         self.forced_prefix = forced_prefix
+        self.prefix_to_counter = {}
 
     def is_name_conflicting(self, name):
         return name in self.existing_names
 
+    def _name_added(self, name):
+        """Callback to alert subclasses when a name has been added.
+
+        .. note::
+
+            This will not get called for the names in the *existing_names*
+            argument to :meth:`__init__`.
+        """
+        pass
+
     def add_name(self, name):
         if self.is_name_conflicting(name):
             raise ValueError("name '%s' conflicts with existing names")
@@ -1753,6 +1897,7 @@ class UniqueNameGenerator:
             raise ValueError("name '%s' does not start with required prefix")
 
         self.existing_names.add(name)
+        self._name_added(name)
 
     def add_names(self, names):
         for name in names:
@@ -1761,22 +1906,45 @@ class UniqueNameGenerator:
     def __call__(self, based_on="id"):
         based_on = self.forced_prefix + based_on
 
-        for var_name in generate_unique_names(based_on):
+        counter = self.prefix_to_counter.get(based_on, None)
+
+        for counter, var_name in generate_numbered_unique_names(based_on, counter):
             if not self.is_name_conflicting(var_name):
                 break
 
+        self.prefix_to_counter[based_on] = counter
+
         var_name = intern(var_name)
 
         self.existing_names.add(var_name)
+        self._name_added(var_name)
         return var_name
 
 # }}}
 
 
+# {{{ recursion limit
+
+class MinRecursionLimit(object):
+    def __init__(self, min_rec_limit):
+        self.min_rec_limit = min_rec_limit
+
+    def __enter__(self):
+        self.prev_recursion_limit = sys.getrecursionlimit()
+        new_limit = max(self.prev_recursion_limit, self.min_rec_limit)
+        sys.setrecursionlimit(new_limit)
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        sys.setrecursionlimit(self.prev_recursion_limit)
+
+# }}}
+
+
 def _test():
     import doctest
     doctest.testmod()
 
+
 if __name__ == "__main__":
     _test()
 
